#ifndef _ufs
#define _ufs

#include "stdint.h"

/*
   Defines for t_headpage.attrflags
*/
#define		ATTR_ISDIR		0x01					//Is this a dir(1) or a file(0)?
#define		ATTR_ISPROT		0x02					//Is this dir/file protected? (FAT compatibility)
#define		ATTR_ISARCH		0x04					//Is this dir/file archived? (FAT compatibility)
#define		ATTR_ISHIDDEN	0x08					//Is this dir/file hidden? (FAT compatibility)
#define		ATTR_ISSYSTEM	0x10					//Is this a system dir/file (FAT compatibility)

/*
   The following structs define the page layout on disk.
   Data is read bytewise into this struct through the sd_read-function
   e.g.: 
     struct t_page page;
     sd_read(adress, &page);
  
   Now this data can be accessed through this structs
   e.g.:
     ....
 	 uint32_t p_nextpage = page.hdr_p_nextpage;
 	 uint32_t curdirentry = page.dirpage.entry[0]
     ....

   Note that the members starting with "hdr_" are direct members of struct t_page.
   The dir entrys, for example, are not direct members, because they are classified
   in the dirpage member.
   That is, because each page (except for diskheader and bitmappage) has the same "hdr_" 
   members at the first 8 byte. 

*/


/*
   A page is a t_headpage, when the content only consists of dir/file information.
   If you go to the next page (if exists), you will find filedata or dir-entrys
   (depending on t_headpage.attrflags & ISDIR)
*/
struct t_headpage
{	
	uint8_t objname[255];							//254-char name of file/dir ('\0' terminated)
	uint8_t attrflags;								//Holds attribute data (look at defines above)
	uint32_t tCreate;								//timestamp of file creation
	uint32_t tLastaccess;							//timestamp of last file access
	uint32_t tLastchange;							//timestamp of last file changing
	uint32_t pParent;								//pointer to parent dir
	uint32_t filesize;								//size of file in byte (directorys have a size of 0)
	uint8_t reserved[228];							//228 bytes reserved
};

/*
   A page is a t_dirpage, when the bit attrflags & ISDIR is set in the headerpage 
   (first page in page-chain)
   This page consists only of 126 pointer to headers of other dirs or files.
   The file/dir pointed on is then a child of this page
*/
struct t_dirpage
{
	uint32_t entry[126];							//holds pointer to linked dir/file. max 126
};

/*
   A page is a t_dirpage, when the bit attrflags & ISDIR is cleared in the headerpage 
   on the previous page.
   This page consists of 504byte raw filedata
*/
struct t_filepage
{
	uint8_t data[504];								//504byte raw filedata
};

struct t_diskheadpage
{
	uint8_t diskname[255];							//Name of disk
	uint32_t disksize;								//size of disk in byte
	uint8_t reserved[245];
};

/*
   Now let us merge all those structs.
   t_page defines the whole page with 8 bytes page-header information and
   the 504byte multi-use space.
*/
struct t_page
{
	struct											//These are the first 8 bytes of each page on disk (except of bitmap and diskheader page)
	{
		uint32_t hdr_pNextpage;						//pointer to next page in chain
		uint16_t hdr_nValidbytes;					//# of valid bytes in this page
		uint16_t hdr_reserved;						//2 bytes reserved
	};
	
	union											//From here, the remaining 504bytes of disk space are defined
	{
		struct t_diskheadpage diskheadpage;
		struct t_headpage headpage;					//Content of page is a header.
		struct t_dirpage dirpage;					//Content of page is a dirlist
		struct t_filepage filepage;					//Content of page is raw filedata
	};
};




#endif //_ufs